package edu.sdsc.sirius.md;

// MBT
import edu.sdsc.mbt.image.*;

import edu.sdsc.sirius.util.*;
import edu.sdsc.sirius.dialogs.*;

// Core
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import java.awt.image.*;
import java.io.*;
import java.awt.print.*;

import javax.imageio.*;
import javax.imageio.spi.*;
import javax.imageio.stream.*;

/**
 * This panel is used to display graph of RMSD change over the course of an MD simulation. The calculation
 * is done in the MDPanel, and the resulting time-value set is passed to this object. If a calculation
 * is performed again and the graph changes, the panel is updated.
 * @author Sasha Buzko
 *
 */
public class RMSDPanel extends JDialog {

	private JPanel imagePanel = new JPanel();
	private JPanel markerPanel = new JPanel();
	private JPanel loadingPanel = new JPanel();
	
	private JDialog panelFrame;
	private JFrame parent;
	private MDPanel mdPanel;
	
	private BufferedImage marker_image;
	private Graphics2D g;

	private BufferedImage m_image;
	private JLabel graphLabel;
	private Graphics2D g2;
		
	private int imageWidth = 500;
	private int imageHeight = 350;
	private int leftMargin = 40;
	private int bottomMargin = 20;
	private int topMargin = 20;
	private int rightMargin = 50;
	
	private boolean loadingVisible = false;
	
	private JLayeredPane pane;
	
	private Object value = new Object();//as a placeholder value for hashes
	
	private MouseInputAdapter mia1;
	private MouseInputAdapter mia2;
	protected static int counter1 = 0;
	protected static int counter2 = 0;
	
	private Book book = null;
	public PageFormat mPageFormat;
	protected int m_maxNumPage = 1;
	
	private double[] scales = { 0.000001, 0.001, 0.005, 0.01, 0.05, 0.1, 0.2, 0.5, 1.0, 5.0, 10.0, 15.0, 25.0, 50 };//allowed scale intervals

	private double[] timescales = { 0.001, 0.001, 0.005, 0.01, 0.1, 0.25, 0.5, 1.0, 2.0, 5.0, 10.0, 25.0, 50.0, 100.0, 250.0, 500.0, 1000.0, 5000.0, 10000.0 };
	
	private Manager callable;
	private JPopupMenu imagePopup;
	
	private double startTime = 0;
	private double endTime = 0;
	
	private double maxEnergy = 0;
	private double minEnergy = 0;
	
	private int pixelStep = 0;//how many pixels per frame advance
	
	private Image marker;
	private JLabel markerLabel;
	
	private Vector rmsdFrames;
	
	private final int colors = 239;
	
	private int currentMarkerPosition = leftMargin;
	
	private int scaleWidth = 0;
	private int scaleHeight = 0;
	
	public RMSDPanel(Manager callable, MDPanel mdPanel){
	
		super((JDialog)mdPanel, "RMSD", false);
		parent = callable.getApplicationFrame();
		this.mdPanel = mdPanel;
		
		this.callable = callable;
		
		//create the image and place it on the image panel
		imagePanel.setBounds(5,0,500,410);
		markerPanel.setBounds(5,0,500,410);
		markerPanel.setOpaque(false);
		
		pane = new JLayeredPane();
		pane.setBounds(0,0,480, 385);
		pane.setPreferredSize(new Dimension(510,385));
	
		JLabel loadingLabel = new JLabel("Chargement...");
		loadingLabel.setHorizontalAlignment(SwingConstants.CENTER);
		loadingLabel.setVerticalAlignment(SwingConstants.CENTER);
		loadingLabel.setFont(new Font("Dialogue", Font.PLAIN, 14));
		loadingPanel.setBounds(5,0,500,410);
		loadingPanel.setLayout(new BorderLayout());
		loadingPanel.add(loadingLabel, BorderLayout.CENTER);
		
		Thread runner = new Thread(){
			public void run(){
				marker = paintMarker();
				ImageIcon icon2 = new ImageIcon(marker);
				markerLabel = new JLabel(icon2);
				markerPanel.add(markerLabel);
			}
		};
		runner.start();
		pane.add(markerPanel, new Integer(20));

		if(!mdPanel.isLoadingCompleted()){
			pane.add(loadingPanel, new Integer(30));
			loadingVisible = true;
		}
		
		panelFrame = this;
		
		//make the graph
		Thread runner2 = new Thread(){
			public void run(){
				paintImage();
			}
		};
		runner2.start();
		
		imagePopup = createImagePopup();
		
		mia1 = new MouseInputAdapter(){
		
			boolean popup = false;
			
			public void mousePressed(MouseEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				checkPopup(e);
			}

			public void mouseReleased(MouseEvent e){
				checkPopup(e);
			}
			
			public void mouseClicked(MouseEvent e){
				checkPopup(e);
			}

			public void checkPopup(MouseEvent e){
				if (e.isPopupTrigger()){
					popup = true;
				}
				start(e, popup);
			}
			
			public void start(MouseEvent e, boolean p){
				counter1++;
				if (counter1 == 3){
					counter1 = 0;
					if (popup){
						imagePopup.show(imagePanel, e.getX(), e.getY());
					}
					popup = false;
				}
			}
			
		};

		pane.add(imagePanel, new Integer(10));
		pane.setVisible(true);
		
		getContentPane().add(pane);
		final MDPanel mp = mdPanel;
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				mp.closeRMSDPanel(true);
			}
		});

		//printing stuff
		PrinterJob pj = PrinterJob.getPrinterJob();
		mPageFormat = pj.defaultPage();
		mPageFormat.setOrientation(PageFormat.LANDSCAPE);
		pj = null;

		//position it right below the main md panel
		Rectangle r = mdPanel.getBounds();
		
		EnergyPanel e = mdPanel.getEnergyPanel();
		if (e != null && e.isVisible()){
			Rectangle er = e.getBounds();
			setBounds(r.x + 100, r.y+r.height+100,515, 385);

		}
		else{
			setBounds(r.x, r.y+r.height,515, 385);
		}
		
		setResizable(false);
		setVisible(true);

	}
	
	public void closePanel(){
		setVisible(false);
		dispose();
	}
	
	public synchronized void paintImage(){
		
		rmsdFrames = mdPanel.getRMSDFrames();
		
		//use the data HashMap to create the image
		m_image = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_BYTE_INDEXED, createColorModel());
		g2 = m_image.createGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);

		scaleWidth = imageWidth - leftMargin - rightMargin;
		scaleHeight = imageHeight - 50;
		
		double timespan = endTime - startTime;
		
		pixelStep = (int)(scaleWidth/(rmsdFrames.size()-1));
		
		int graphHeight = scaleHeight - 30;
		
		//draw the background
		g2.setColor(Color.white);
		g2.fillRect(0,0,imageWidth, imageHeight);
		g2.setColor(Color.black);
		g2.drawRect(0,0,imageWidth - 1, imageHeight - 1);
		
		//draw the axes
		g2.drawLine(leftMargin, topMargin, leftMargin, topMargin + scaleHeight);
		g2.drawLine(leftMargin, topMargin + scaleHeight, leftMargin + scaleWidth, topMargin + scaleHeight);
		
		
		//draw percentage scale for time
		g2.setFont(new Font("Helvetica", Font.PLAIN, 10));

		//first, check whether there is any time information, i.e. whether we are reading an out file
		Vector data = mdPanel.getDataFrames();
		if (data != null && data.size() > 0){
			//we can draw time scale using ps
			//read the last frame to get the final timestamp
			double[] d = (double[])data.get(data.size()-1);
			double endTime = d[1];
			
			d = (double[])data.get(0);
			double startTime = d[1];
			
			double ratio = (endTime - startTime)/scaleWidth;
			
			double interval = 0;
			for (int i = timescales.length-1; i >= 0; i--){
				double sratio = (endTime - startTime)/timescales[i];
				if (sratio > 5){
					interval = timescales[i];
					break;
				}
			}
			
			int ir = (int)(interval/ratio);
			
			g2.drawString(StringUtils.getFormattedNumber(startTime, 1), leftMargin-5, topMargin+scaleHeight+20);

			//draw the scale
			int sx = leftMargin + ir;
			int sy = topMargin+scaleHeight;
			int counter = 1;
			int limit = leftMargin + scaleWidth;
			while (true){
				if (sx > limit) break;
				g2.drawLine(sx, topMargin, sx, sy+5);
				double vv = startTime + interval*counter;
				String mark = null;
				if (interval > 1){
					mark = StringUtils.getFormattedNumber(vv, 0);
				}
				else if (interval > 0.05){
					mark = StringUtils.getFormattedNumber(vv, 1);
				}
				else if (interval < 0.01){
					mark = StringUtils.getFormattedNumber(vv, 3);
				}
				else{
					mark = StringUtils.getFormattedNumber(vv, 2);
				}
				g2.drawString(mark, sx-5, sy+20);
				
				sx += ir;
				counter++;
			}
			
			//draw the ps string 
			g2.drawString("ps", leftMargin+scaleWidth+25, topMargin+scaleHeight+20);
			
		}
		else{
			//draw a percentage scale, since time is not available
			int singlespan = (int)(scaleWidth/5);
			for (int i = 0; i <= 5; i++){//six tick marks
				int x = leftMargin + i*singlespan;
				if (i == 5){
					x = leftMargin + scaleWidth;
				}
				g2.drawLine(x,topMargin,x, topMargin+scaleHeight+5);
				int mark = i*20;
				String text = new Integer(mark).toString() + "%";
				if (i == 5){
					g2.drawString(text, x-15, topMargin+scaleHeight+20);
				}
				else{
					g2.drawString(text, x-7, topMargin+scaleHeight+20);
				}
			}
		}
		
		
		g2.drawString("RMSD, A", 5, 15);
		
		//get the vertical scale
		double[] values = new double[rmsdFrames.size()];
		double max = 0;
		
		for (int i = 0; i < rmsdFrames.size(); i++){
			double v = ((Double)rmsdFrames.get(i)).doubleValue();
			if (v > max) max = v;
			values[i] = v;
		}
		
		double ratio = max/graphHeight;//how many rmsd units per vertical pixel
		
		//draw the zero line
		g2.drawLine(leftMargin, topMargin+scaleHeight, leftMargin-10, topMargin+scaleHeight);
		g2.drawString("0", leftMargin - 20, topMargin+scaleHeight+4);
		
		int zerolevel = topMargin + scaleHeight;
		
		g2.setColor(Color.BLACK);
		
		//draw the graph itself
		if (pixelStep > 0){
			Point lastValue = new Point(-1,-1);
			for (int i = 0; i < values.length; i++){
				double v = values[i];
				int x = leftMargin + i*pixelStep;
				int y = zerolevel - (int)(v/ratio);
				if (lastValue.x > -1){
					g2.drawLine(lastValue.x,lastValue.y, x, y);
				}
				
				lastValue.x = x;
				lastValue.y = y;
				
			}
		}
		else{
			double stepRatio = (double)rmsdFrames.size()/(double)scaleWidth;
			//number of steps is larger than number of available pixels, so steps must be skipped
			//in this case, walk along pixels (instead of values), and compute the corresponding value for the given pixel
			Point lastValue = new Point(-1,-1);
			for (int i = 0; i < scaleWidth; i++){
				int currentFrame = (int)(stepRatio*i + 0.9);//to force rounding up
				
				double v = values[currentFrame];
				int x = leftMargin + i;
				int y = zerolevel - (int)(v/ratio);
				if (lastValue.x > -1){
					g2.drawLine(lastValue.x,lastValue.y, x, y);
				}
				
				lastValue.x = x;
				lastValue.y = y;
			}
			
			
			
		}
		
//		System.out.println("max = " + max);
		//find out which scale to use
		double interval = 0;
		for (int i = scales.length-1; i >= 0; i--){
			double sratio = max/scales[i];
			if (sratio > 5){
//				System.out.println("sratio = " + sratio);
				interval = scales[i];
				break;
			}
		}
		
		//draw the scale
		int sx = leftMargin;
		int sy = topMargin+scaleHeight-((int)(interval/ratio));
		int counter = 1;
		while (true){
			if (sy < topMargin+10) break;
			g2.drawLine(sx, sy, sx-5, sy);
			double vv = interval*counter;
			String mark = null;
			if (interval > 0.05){
				mark = StringUtils.getFormattedNumber(vv, 1);
			}
			else if (interval < 0.01){
				mark = StringUtils.getFormattedNumber(vv, 3);
			}
			else{
				mark = StringUtils.getFormattedNumber(vv, 2);
			}
			g2.drawString(mark, 5,sy+5);
			
			sy -= (int)(interval/ratio);
			counter++;
		}

		ImageIcon icon = new ImageIcon(m_image);
		graphLabel = new JLabel(icon);
		graphLabel.addMouseListener( mia1 );
		graphLabel.addMouseMotionListener( mia1 );
		imagePanel.add(graphLabel);
		
		if (loadingVisible){
			pane.remove(loadingPanel);
			loadingVisible = false;
			pane.revalidate();
			pane.repaint();
		}
		pane.revalidate();
		pane.repaint();
		
	}
	
	public synchronized Image paintMarker(){
		
		//use the data HashMap to create the image
		marker_image = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_BYTE_INDEXED, createColorModel());
		g = marker_image.createGraphics();
		
		return marker_image;
	}
	
	public void updateMarker(int frame){
		
		if (frame == 1) frame = 0;
		
		if (frame < 1){
			if (g != null){
				g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 0.0f ) );
				g.setColor(Color.white);
				g.drawLine(currentMarkerPosition, topMargin, currentMarkerPosition, topMargin+scaleHeight);
				markerPanel.repaint();
			}
			return;
		}

		if (markerPanel == null){
			try{
				markerPanel.remove(markerLabel);
			}
			catch (Exception e){}
			marker = paintMarker();
			ImageIcon icon2 = new ImageIcon(marker);
			markerLabel = new JLabel(icon2);
			markerPanel.add(markerLabel);
			currentMarkerPosition = leftMargin;
			markerPanel.revalidate();
			markerPanel.repaint();
		}
		else{
			//simply update the marker line
			//draw over the last line
			g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 0.0f ) );
			g.setColor(Color.white);
			g.drawLine(currentMarkerPosition, topMargin, currentMarkerPosition, topMargin+scaleHeight);
			
			int x = 0;
			
			if (pixelStep > 0){
				x = leftMargin + (frame-1)*pixelStep;
			}
			else{
				double stepRatio = (double)rmsdFrames.size()/(double)scaleWidth;
				x = leftMargin + (int)((frame-1)/stepRatio);
			}
			
			g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 1.0f ) );
			g.setColor(Color.red);
			g.drawLine(x, topMargin, x, topMargin+scaleHeight);
			currentMarkerPosition = x;
			
			markerPanel.repaint();
		}
		
	}
	
	/**
	 * This method is called when RMSD computation method is changed, and values and scale have to be recalculated
	 *
	 */
	public void updateGraph(int set, String name){
		
		try{
			imagePanel.remove(graphLabel);
		}
		catch (Exception e){
			e.printStackTrace();
		}
		
		try{
			paintImage();
			
			if (set == MDRMSDDialog.USE_ALL_ATOMS){
				setTitle("RMSD: All atoms");
			}
			else if (set == MDRMSDDialog.USE_ALL_EXCEPT_SOLVENT){
				setTitle("RMSD: All atoms except solvent");
			}
			else if (set == MDRMSDDialog.USE_ALL_HEAVY_ATOMS_EXCEPT_SOLVENT){
				setTitle("RMSD: Heavy atoms except solvent");
			}
			else if (set == MDRMSDDialog.USE_BACKBONE_ATOMS){
				setTitle("RMSD: Backbone atoms");
			}
			else if (set == MDRMSDDialog.USE_CA_ATOMS){
				setTitle("RMSD: Alpha carbons in protein backbone");
			}
			else if (set == MDRMSDDialog.USE_DEFINED_SET){
				setTitle("RMSD: Custom atom set " + name);
			}
			else{
				setTitle("RMSD");
			}
			
			
			pane.revalidate();
			pane.repaint();
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public JPopupMenu createImagePopup(){
	
		JPopupMenu popup = new JPopupMenu();
		
		JMenuItem blank = new JMenuItem("");
		blank.setEnabled(false);
		JMenuItem saveData = new JMenuItem("Sauvegarde de donnes RMSD...");
		saveData.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				saveData();
				repaint();
			}
				
		});
		
		JMenuItem save = new JMenuItem("Enregistrer l\'image...");
		save.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				saveImage();
				repaint();
			}
				
		});
		
		
		JMenuItem printSetup = new JMenuItem("Initialisation de l\'impression...");
		printSetup.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				printSetupHandler();
			}
		});
		JMenuItem print = new JMenuItem("Imprimer l\'image...");
		print.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				printImage();
			}
		});
		
		JMenuItem close = new JMenuItem("Fermer");
		close.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				mdPanel.closeRMSDPanel(true);
			}
		});
		
		popup.add(blank);
		popup.add(saveData);
		popup.add(save);
		popup.add(new JSeparator());
		popup.add(printSetup);
		popup.add(print);
		popup.add(new JSeparator());
		popup.add(close);
		 
		return popup;
	}
		
	public void saveImage(){

		try{
			ImageHandler.saveImage(callable, m_image);
		}
		catch (IOException e){
			JOptionPane.showMessageDialog(this, "Erreur d\'criture du fichier :" + e, "Erreur", JOptionPane.ERROR_MESSAGE);
		}

	}
	
	public void saveData(){
		if (rmsdFrames == null || rmsdFrames.size() == 0) return;
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(callable.getLastUsedDirectory());
		chooser.setDialogTitle("Sauvegarde de donnes RMSD");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);

		chooser.addChoosableFileFilter( new FileNameFilter("txt", "Format texte dlimit par des espaces") );
		chooser.addChoosableFileFilter( new FileNameFilter("txt", "Format texte dlimit par des Tab") );
		chooser.addChoosableFileFilter( new FileNameFilter("csv", "Format CSV sparations par des virgules") );
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(callable.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		
		File filename = chooser.getSelectedFile();
		
		callable.setLastUsedDirectory(filename.getParent());
		
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		FileFilter filter = chooser.getFileFilter();
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}

		StringBuffer buffer = new StringBuffer();
		if (filter.getDescription().startsWith("Espace")){
			for (int i = 1; i <= rmsdFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append(" ");
				double v = ((Double)rmsdFrames.get(i-1)).doubleValue();
				buffer.append(StringUtils.getFormattedNumber(v, 4));
				buffer.append("\n");
			}
		}
		else if (filter.getDescription().startsWith("Tab")){
			for (int i = 1; i <= rmsdFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append("\t");
				double v = ((Double)rmsdFrames.get(i-1)).doubleValue();
				buffer.append(StringUtils.getFormattedNumber(v, 4));
				buffer.append("\n");
			}
		}
		else if (filter.getDescription().startsWith("Virgule")){
			for (int i = 1; i <= rmsdFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append(",");
				double v = ((Double)rmsdFrames.get(i-1)).doubleValue();
				buffer.append(StringUtils.getFormattedNumber(v, 4));
				buffer.append("\n");
			}
		}
		else{
			callable.displayErrorMessage("Option de format de fichier illgale");
			return;
		}
		
		try{
			FileWriter fr = new FileWriter(fname);
			PrintWriter printer = new PrintWriter(fr);
			printer.print(buffer.toString());
			printer.close();
		}
		catch (Exception e){
			e.printStackTrace();
			
		}
		
	}
	
	public void printImage(){
		
		//print the currently displayed image
		
		Thread t = new Thread(){
			public void run(){

				PrinterJob pj = PrinterJob.getPrinterJob();

				//set the cursor, 'cause the wait will be long
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				book = new Book();
				
				//get image dimensions
				int iWidth = m_image.getWidth();
				int iHeight = m_image.getHeight();
				double aspectRatio = imageWidth/imageHeight;
				
				//get the page format
				int pageWidth = (int)mPageFormat.getImageableWidth();
				int pageHeight = (int)(mPageFormat.getImageableHeight()*0.9);//make it a bit more legible by compression


				//now with this image, create a Printable instance and add it to Book
				PagePrintable p = new PagePrintable(m_image, pageWidth, pageHeight, iWidth, iHeight);

				book.append(p, mPageFormat);




				pj.setPageable(book);
				if (pj.printDialog()){
					try{
						pj.print();
					}
					catch (PrinterException pe){
						System.out.println("Erreur " + pe);
						JOptionPane.showMessageDialog(panelFrame, "Erreur d\'impression" + pe, "Erreur", JOptionPane.ERROR_MESSAGE);
					}
				}

				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				book = null;
			}
		};
		
		t.start();
		
		
	}			
	
	
	public void printSetupHandler(){

		//this method is called from Viewer when Print Setup is selected
		//currently active frame is found and this method is called

		Thread t = new Thread(){
			public void run(){
				PrinterJob pj = PrinterJob.getPrinterJob();
				mPageFormat = pj.pageDialog(mPageFormat);
			}
		};
		t.start();
	}
	
	
	private void savePrint( BufferedImage image, String filename ){

		try
		{
			File file = new File("D:\\print" + filename + ".png");
			// Get the currently-selected item in the combo box
			FormatDescriptor fd =
				(FormatDescriptor) (getWriterFormats().elementAt(2));

			// Get the ImageWriter stored in this FormatDescriptor
			ImageWriter imageWriter = fd.imageWriter;

			// Get an output stream for the file
			FileOutputStream fout = new FileOutputStream( file );

			// Turn it into an ImageOutputStreamn
			ImageOutputStream ios = ImageIO.createImageOutputStream( fout );

			// Plug this stream into the ImageWriter
			imageWriter.setOutput( ios );

			// Write the image
			imageWriter.write( image );

			// Dont forget to close the file!
			fout.close();
		}
		catch( IOException ie )
		{
			System.out.println("Impossible d\'enregistrer l\'image");
			ie.printStackTrace( );
		}
	}
	
	static private Vector getWriterFormats( )
	{
		Vector formats = new Vector( );
		Hashtable seen = new Hashtable( );

		String names[] = ImageIO.getWriterFormatNames( );
		for ( int i=0; i<names.length; ++i )
		{
			String name = names[i];
			Iterator writers = ImageIO.getImageWritersByFormatName( name );
			while ( writers.hasNext() )
			{
				ImageWriter iw = (ImageWriter) writers.next( );
				ImageWriterSpi iws = iw.getOriginatingProvider( );
				String suffixes[] = iws.getFileSuffixes( );
				boolean allseen = true;
				for ( int j=0; j<suffixes.length; ++j )
				{
					String suffix = suffixes[j];
					suffix = suffix.toLowerCase( );
					if ( ! seen.containsKey( suffix ) )
					{
						seen.put( suffix, suffix );
						String description = name + " (*." + suffix + ")";
						FormatDescriptor fd =
							new FormatDescriptor( suffix, iw, description );
						formats.addElement( fd );
					}
				}
			}
		}
	

		return formats;
	}

		
	static private class FormatDescriptor
	{
		public String suffix;
		public ImageWriter imageWriter;
		public String description;

		public FormatDescriptor( String suffix, ImageWriter imageWriter,
			String description )
		{
			this.suffix = suffix;
			this.imageWriter = imageWriter;
			this.description = description;
		}

		public String toString( )
		{
			return description;
		}
	}
	
	public IndexColorModel createColorModel() {

		byte[] red   = { (byte)255,(byte)255,0,0,(byte)255,0,(byte)255,0,(byte)240,25,(byte)232,(byte)155,20,(byte)161,43,(byte)229,(byte)229,20,(byte)252,5,(byte)255,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,0,25 };
		byte[] green  = { (byte)255,0,(byte)255,0,0,(byte)255,(byte)255,100,(byte)240,15,123,(byte)155,50,43,(byte)161,(byte)161,43,(byte)140,93,5,(byte)255,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,0,25 };
		byte[] blue = { (byte)255,0,0,(byte)255,(byte)255,(byte)255,0,0,25,100,20,(byte)155,(byte)170,(byte)229,(byte)229,43,(byte)161,20,100,5,(byte)255,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,0,25 };
		byte[] alpha = { 0,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255};

		IndexColorModel cm = new IndexColorModel(8, colors, red, green, blue, alpha);

		red = null;
		green = null;
		blue = null;
		alpha = null;

		return cm;
	}

	
}


